"""
Date: 31st August 2018
Some functions to make the reading of json and csv files easy and no nonsense. The following functions are available:
1. readCsvReturnList: read a csv file path and return a list.
2. readJsonReturnData: Read a json file and return a dictionary.
3. readOnlineCsvReturnList: Read a csv file hosted somewhere online and return a list.
4. readOnlineJsonReturnData: Read a json file hosted somwehere online and return a dict.

"""

# coding=utf-8
import csv,json
import os,tempfile
import urllib.request, urllib.parse, urllib.error
import  logging
from typing import List,Dict

logger=logging.getLogger("__main__")

#Need the header for those pages
_hdr = {'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.11 (KHTML, like Gecko) Chrome/23.0.1271.64 Safari/537.11',
       'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
       'Accept-Charset': 'ISO-8859-1,utf-8;q=0.7,*;q=0.3',
       'Accept-Encoding': 'none',
       'Accept-Language': 'en-US,en;q=0.8',
       'Connection': 'keep-alive'}

def _downloadUrlToFile(urlLink,filePath):
    """

    :param url: Url to be downloaded. Will usually be a linke to a json, csv file.
    :param filePath: Path to write to.
    :return:
    """
    try:

        csvRequest=urllib.request.Request(urlLink,headers=_hdr)
        csvData=urllib.request.urlopen(csvRequest).read()

        with open(filePath,"wb") as csvWrite:
            csvWrite.write(csvData)
        logger.debug("Downloaded %s to %s"%(urlLink,filePath))
    except:
        logger.error("Could not download %s"%urlLink)
        raise Exception

    return filePath

#TODO: Create a function to summarize jsons. Like print a summary of what all are inside.
def readCsvReturnList(csvFilePath):
    """
    Take a csv file path and return a List.
    :param csvFilePath: Path of the csv file.
    :return: A list extracted from the csv.
    :rtype: List
    """
    assert os.path.exists(csvFilePath),"The input for csvFilePath(%s) was not found."%csvFilePath


    with open(csvFilePath,encoding="utf8") as csvData:
        outputList= list(csv.reader(csvData))
    logger.debug("Successfully extracted data from %s"%csvFilePath)

    return outputList

def readJsonReturnData(jsonFilePath):
    """
    Read a json file and return a dict.
    :param jsonFilePath:
    :return:
    :rtype: Dict
    """

    assert os.path.exists(jsonFilePath),"The input for jsonFilePath(%s) was not found."%jsonFilePath

    with open(jsonFilePath,encoding="utf8") as jsonData:
        outputDict=json.load(jsonData)

    return outputDict

def readOnlineCsvReturnList(csvUrl):
    """
    Return list from
    :param csvUrl:
    :return:
    :rtype: List
    """

    tmpCsvFile=tempfile.mktemp(suffix=".csv")

    _downloadUrlToFile(csvUrl,tmpCsvFile)

    listVal= readCsvReturnList(tmpCsvFile)

    os.remove(tmpCsvFile)
    logger.debug("Deleting the temp csv file %s"%tmpCsvFile)

    return  listVal

def readOnlineJsonReturnData(jsonUrl):
    """
    Return list from
    :param jsonUrl:
    :return:
    """

    tmpJsonFile=tempfile.mktemp(suffix=".json")

    tmpJsonFile=_downloadUrlToFile(jsonUrl,tmpJsonFile)

    jsonVal= readJsonReturnData(tmpJsonFile)


    os.remove(tmpJsonFile)

    return  jsonVal


if __name__ =="__main__":


    print((readOnlineCsvReturnList(r"https://raw.githubusercontent.com/haliaeetus/iso-639/master/data/iso_639-1.csv")))

    print((readOnlineJsonReturnData("http://lighting-materials.com/materials.json")))
