"""
Date: 24 August 2018

Contains a class called LoggerMain that is a all-in-one solution for logging. See usage.
"""
import logging


class LoggerMain(logging.Logger):
    """
    This class was written to allow a one line implementation of a logging session. At its core, it inherits from
    logging.Logger and then copies the entire dict from logging.getLogger("__main__").

    It is meant to be used in the main script when I am playing around with other scripts.

    Simple usage is something like this:
        from loggerMain import LoggerMain

        logger=LoggerMain() #And thats all!!

        #Then subsequent calls can be made just like a normal logger.
        logger.debug("What?!")

    Customizing the logger with options from the StringFormats class.

        from loggerMain import LoggerMain

        from fmtStrings import StringFormats

        logger=LoggerMain(format=StringFormats.timeLvlModlFuncMsg)


    A screen logger is assigned by default. File handler can be assigned if needed. Similarly formatting is assigned
    as well.
        """
    __initialized=False
    "This attribute will be set to True once a single instance of this class has been instantiated. For a given session," \
    "only instance should be used."

    def __init__(self, name="__main__",logFile=None,logToScreen=True,format='%(asctime)s "%(levelname)s"|mod:%(module)s|func:%(funcName)s|msg:%(message)s',
                 timeFormat="%H:%M:%S",level=logging.DEBUG,logFileMode="a",mainCheck=True):
        """
        Initialize a logger with the following parameters. DO NOT change "__main__" !
        :param name: Name of the logger. Do not change this one.!!! Using "__main__"  across all scripts allows me
            to use the same settings across all of my scripts.
        :param logFile: Name of a filepath to log to. Defaults to None, ie no logging to a file.
        :param logToScreen: Defaults to True, ie will log to a screen. It might useful to turn this off in situations
            where I am trying to log but also get pretty print outputs.
        :param format: Format string for formatter. Defaults to '%(asctime)s |md:%(module)s |fun%(funcName)s |msg%(message)s'.
            Note that this can also be set using the StringFormats container class in fmtStrings module.
        :param timeFormat: Time format for formatter . Defaults to hour minute second. "%H:%M:%S". Note that this
             can also be set using the TimeFormats container class in fmtStringsTime module.
        :param level: Logging level. Defaults to DEBUG.
        :param logFileMode: Defaults to a (for append).
        :param Throw a warning in case this class is being instantiated from a non main script. This class should
            not be instantiated thorugh a non main script as there would be no control over the parameters of the main
            logger then.
        """
        #24 August 2018: Not sure if this assigning dicts is safe, but it seems to be working (for now!)
        logging.Logger.__init__(self,name)
        self.__dict__=logging.getLogger(name).__dict__
        formatter=logging.Formatter(format,timeFormat)

        self.setLevel(level)

        if logToScreen:
            self.defaultScreenLogger=logging.StreamHandler()
            self.defaultScreenLogger.setFormatter(formatter)
            self.addHandler(self.defaultScreenLogger)
        else:
            self.defaultScreenLogger=None

        if logFile:
            self.defaultFileLogger = logging.FileHandler(logFile,mode=logFileMode)
            self.defaultFileLogger.setFormatter(formatter)
            self.addHandler(self.defaultFileLogger)
        else:
            self.defaultScreenLogger = None

        if mainCheck:
            if LoggerMain.__initialized:
                import warnings
                warnings.warn("\n\tThe '__main__' logger has already been initiated once. Initiating again is a huge PITA!!\n"*3)
            else:
                LoggerMain.__initialized=True

    def setLevel_DEBUG(self):
        """Assign the said level to the logger. This function was written to set Levels through a method call
        than do something like this:
            logger.setLevel(logging.SOMELEVEL)
        """
        self.setLevel(logging.DEBUG)

    def setLevel_WARNING(self):
        """Assign the said level to the logger. This function was written to set Levels through a method call
        than do something like this:
            logger.setLevel(logging.SOMELEVEL)
        """
        self.setLevel(logging.WARNING)

    def setLevel_ERROR(self):
        """Assign the said level to the logger. This function was written to set Levels through a method call
        than do something like this:
            logger.setLevel(logging.SOMELEVEL)
        """
        self.setLevel(logging.ERROR)

    def setLevel_INFO(self):
        """Assign the said level to the logger. This function was written to set Levels through a method call
        than do something like this:
            logger.setLevel(logging.SOMELEVEL)
        """
        self.setLevel(logging.INFO)

    def setLevel_CRITICAL(self):
        """Assign the said level to the logger. This function was written to set Levels through a method call
        than do something like this:
            logger.setLevel(logging.SOMELEVEL)
        """
        self.setLevel(logging.CRITICAL)

    def setLevel_NOTSET(self):
        """Assign the said level to the logger. This function was written to set Levels through a method call
        than do something like this:
            logger.setLevel(logging.SOMELEVEL)
        """
        self.setLevel(logging.NOTSET)


if __name__ =="__main__":
    pass