# coding=utf-8

import datetime,time

class CountTime(object):
    """A simplistic counter for keeping track of time.

    Usage:
        x = CountTime()
        x.start()
        time.sleep(10)
        x.stop()

        #For counting seconds.
        print((x.countSeconds()))

        #For counting minutes.
        print((x.countMinutes()))

    """
    def __init__(self):
        """A counter will start at instantiation. This can be overridden by using the start()
         method. """
        self.startTime = time.time()
        self.stopTime = None
    def start(self):
        """When called it will start the timer."""
        self.startTime = time.time()

    def stop(self):
        """Stop the clock."""
        self.stopTime = time.time()

    def countSeconds(self,returnInt=True):
        """Return the number of seconds lapsed."""
        assert self.stopTime,'The counter has not been stopped yet. Call the stop() method' \
                             'to stop the timer.'
        seconds = round(self.stopTime-self.startTime)
        if returnInt:
            seconds = int(seconds)
        return seconds


    def countMilliSeconds(self,returnInt=True):
        """Return the number of seconds lapsed."""
        assert self.stopTime,'The counter has not been stopped yet. Call the stop() method' \
                             'to stop the timer.'
        seconds = round((self.stopTime-self.startTime)*1000,3)
        if returnInt:
            seconds = int(seconds)
        return seconds

    def countMicroSeconds(self,returnInt=True):
        """Return the number of seconds lapsed."""
        assert self.stopTime,'The counter has not been stopped yet. Call the stop() method' \
                             'to stop the timer.'
        seconds = round((self.stopTime-self.startTime)*1000000,3)
        if returnInt:
            seconds = int(seconds)
        return seconds

    def countMinutes(self,roundTo=2,overRideRounding=True):
        """Return minutes. The values are rounded to 2 decimal places."""

        seconds = self.countSeconds()
        if seconds < 60 and overRideRounding:
            roundTo = 3
        return round(seconds/60,roundTo)

    def countHours(self,roundTo=3,overRideRounding=True):
        """Return hours."""
        seconds = self.countSeconds()

        if seconds < 60 and overRideRounding:
            roundTo = 4

        return round(seconds / 3600, roundTo)


    def countString(self,separator='_'):
        """Return a pretty string in HourMinuteSeconds format."""
        timeStr= str(datetime.timedelta(seconds=self.countSeconds()))
        if separator != ':':
            timeStr = timeStr.replace(':',separator)

        return timeStr

    def __str__(self):
        """Returns the output from countString method."""
        return self.countString()


if __name__ == "__main__":
    pass